<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_conveythis
 *
 * @copyright   Copyright (C) 2018 www.conveythis.com, All rights reserved.
 * @license     ConveyThis Translate is licensed under GPLv2 license; see LICENSE.txt
 */
defined( '_JEXEC' ) or die;

$title = JText::_('ConveyThis');

JToolBarHelper::title( $title, 'conveythis' );

$document = JFactory::getDocument();
$document->setTitle( $title );

$document->addStyleSheet('/media/conveythis/css/semantic.min.css');
$document->addStyleSheet('/media/conveythis/css/range.css');

$document->addScript('/media/conveythis/js/jquery-3.3.1.min.js');
$document->addScript('/media/conveythis/js/semantic.min.js');
$document->addScript('/media/conveythis/js/range.js');

$document->addScript('//cdn.conveythis.com/js/17/conveythis-admin-17.js');
$document->addScript('//cdn.conveythis.com/js/17/conveythis-17.js');
$document->addScript('//cdn.conveythis.com/js/17/conveythis-view-17.js');


if( file_exists( JPATH_PLUGINS . '/system/conveythis/conveythis.php' ) )
{
	$plugin = JPluginHelper::getPlugin('system', 'conveythis');

	if( $plugin )
	{
		$params = new JRegistry( $plugin->params );

		$app = JFactory::getApplication();
		$input = $app->input->post->getArray();

		if( !empty( $input ) )
		{
			if( empty( $input['api_key'] ) ) $input['api_key'] = '';

			$params->set( 'api_key', $input['api_key'] );

			if( !empty( $input['source_language'] ) )
			{
				$params->set( 'source_language', $input['source_language'] );
			}

			if( !empty( $input['target_languages'] ) )
			{
				$target_languages = explode( ',', $input['target_languages'] );
				$params->set( 'target_languages', $target_languages );
			}

			if( !empty( $input['style_change_language'] ) )
			{
				$params->set( 'style_change_language', $input['style_change_language'] );
			}

			if( !empty( $input['style_change_flag'] ) )
			{
				$params->set( 'style_change_flag', $input['style_change_flag'] );
			}

			if( !empty( $input['style_flag'] ) )
			{
				$params->set( 'style_flag', $input['style_flag'] );
			}

			if( !empty( $input['style_text'] ) )
			{
				$params->set( 'style_text', $input['style_text'] );
			}

			if( !empty( $input['style_position'] ) )
			{
				$params->set( 'style_position', $input['style_position'] );
			}

			if( !empty( $input['style_indenting'] ) )
			{
				$params->set( 'style_indenting', $input['style_indenting'] );
			}

			if( !empty( $input['alternate'] ) )
			{
				$params->set( 'alternate', $input['alternate'] );
			}

			$extension = JTable::getInstance( 'extension' );
			$extension->load( array('element' => 'conveythis') );
			$extension->bind( array(
				'params' => $params->toString()
			));

			if (!$extension->check()) {
				$this->setError($extension->getError());
				return false;
			}
			if (!$extension->store()) {
				$this->setError($extension->getError());
				return false;
			}
		}

		JPluginHelper::importPlugin( 'system', 'conveythis' );
		$dispatcher = JEventDispatcher::getInstance();

		list( $languages ) = $dispatcher->trigger( 'getLanguages' );
		list( $flags ) = $dispatcher->trigger( 'getFlags' );

		$account;
		$language_code = '';
		$referrer = '//' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
		$error = array();

		$api_key = $params->get( 'api_key', '' );
		$source_language = $params->get( 'source_language', '' );
		$target_languages = $params->get( 'target_languages', array() );
		$style_change_language = $params->get( 'style_change_language', array() );
		$style_change_flag = $params->get( 'style_change_flag', array() );
		$style_flag = $params->get( 'style_flag', 'rect' );
		$style_text = $params->get( 'style_text', 'full-text' );
		$style_position = $params->get( 'style_position', 'right-bottom' );
		$style_indenting = $params->get( 'style_indenting', '0' );
		$alternate = $params->get( 'alternate', '1' );

		if( !empty( $api_key ) )
		{
			list( $account ) = $dispatcher->trigger( 'stripApi', array( '/user/account/init/', array(
				'dataType' => 'serialize',
				'api_key' => $api_key,
				'referrer' => $referrer,
				'source_language' => $source_language,
				'target_languages' => $target_languages,
			)));

			if( !empty( $account->error ) )
			{
				$error['message'] = $account->error;
			}
		}

		else
		{
			$error['message'] = 'ConveyThis plugin installed but not yet configured.';
		}
		require_once('settings.php');
	}

	else
	{
		echo '<p>ConveyThis plugin not published.</p>';
		echo '<p>Please go to the "Extensions / Plugins" and enable ConveyThis plugin.</p>';
	}
}

else
{
	echo '<p>ConveyThis plugin not installed.</p>';
	echo '<p>Please download and install the ConveyThis plugin with the Joomla! Extensions Directory.</p>';
}
