<?php
/**
 * @package     Joomla.Plugin
 * @subpackage  System.Conveythis
 *
 * @copyright   Copyright (C) 2018 www.conveythis.com, All rights reserved.
 * @license     ConveyThis Translate is licensed under GPLv2 license; see LICENSE.txt
 */
defined('_JEXEC') or die('Restricted access');

class Parser {

	var $buffer = '';
	var $segments = array();
	var $tags = array();
	var $index = 0;
	var $pid = array();
	var $data = array();
	var $QUOTE_HELPER;
	var $TAG_HELPER;

	var $SYMBOL_HORIZONTAL_TAB;
	var $SYMBOL_LINE_FEED;
	var $SYMBOL_CARRIAGE_RETURN;
	var $SYMBOL_SPACE;
	var $SYMBOL_DOUBLE_QUOTES;
	var $SYMBOL_SINGLE_QUOTE;
	var $SYMBOL_SLASH;
	var $SYMBOL_LEFT_QUOTE;
	var $SYMBOL_EQUALS;
	var $SYMBOL_RIGHT_QUOTE;


	function __construct( $string )
	{
		$this->SYMBOL_HORIZONTAL_TAB	= chr( 9 );
		$this->SYMBOL_LINE_FEED			= chr( 10 );
		$this->SYMBOL_CARRIAGE_RETURN	= chr( 13 );
		$this->SYMBOL_SPACE				= chr( 32 );
		$this->SYMBOL_DOUBLE_QUOTES		= chr( 34 );
		$this->SYMBOL_SINGLE_QUOTE		= chr( 39 );
		$this->SYMBOL_SLASH				= chr( 47 );
		$this->SYMBOL_LEFT_QUOTE		= chr( 60 );
		$this->SYMBOL_EQUALS			= chr( 61 );
		$this->SYMBOL_RIGHT_QUOTE		= chr( 62 );

		$this->read( $string );
	}

	function bufferReset()
	{
		$this->buffer = '';
	}

	function addAttrValue( $value )
	{
		$trim_value = trim( $value );

		if( !empty( $trim_value ) && $trim_value != '#' )
		{
			$index = sizeof( $this->data[$this->index]['attributes'] ) - 1;
			$this->data[$this->index]['attributes'][$index][1] = $value;
			$this->data[$this->index]['attributes'][$index][2] = $this->QUOTE_HELPER;
		}
	}

	function addAttribute( $value )
	{
		$temp = trim( $value );

		if( !empty( $temp ) )
		{
			$this->data[$this->index]['attributes'][] = array( $value, '' );
		}
	}

	//

	function addSegments()
	{
		$i = sizeof( $this->segments );

		$start = false;

		while( $i > 0 )
		{
			$temp = array_pop( $this->segments );
			$this->data[$this->index]['segments'][] = $temp[1];

			if( $this->index == $temp[0] )
			{
				break;
			}
			$i--;
		}
	}

	function prepareSegment( $value )
	{
		$temp = trim( $value );

		if( !empty( $temp ) && !is_numeric( $temp ) )
		{
			$this->segments[] = array( $this->index, $value );
		}
	}

	//

	function addTag( $value )
	{
		if( $this->THIS_OPENING_TAG )
		{
			if( $this->ifSegment( $value ) )
			{
				$this->TAG_HELPER = "/".$value;
			}
			$this->index = sizeof( $this->data );
			$this->data[] =	array(
				'name' => $value,
				'path' => implode( '.', $this->pid ),
			);
			array_push( $this->tags, array( $this->index, $value ) );
			array_push( $this->pid, $this->index );
		}
		
		else
		{
			if( $this->ifSegment( $value ) )
			{
				$this->TAG_HELPER = null;
			}
			$i = sizeof( $this->tags );

			while( $i > 0 )
			{
				$temp = array_pop( $this->tags );

				if( $value == $temp[1] )
				{
					$this->index = $temp[0];
					array_pop( $this->pid );
					break;
				}
				$i--;
			}
			$this->addSegments();
		}
	}

	function ifSpace( $value )
	{
		if( $value === $this->SYMBOL_SPACE ||
			$value === $this->SYMBOL_HORIZONTAL_TAB ||
			$value === $this->SYMBOL_CARRIAGE_RETURN ||
			$value === $this->SYMBOL_LINE_FEED )
		{
			return true;
		}
		return false;
	}

	function ifEquals( $a, $b )
	{
		if( strcasecmp( $a, $b ) === 0 )
		{
			return true;
		}
		return false;
	}

	function ifSegment( $value )
	{
		if( $this->ifEquals( $value, 'script' ) || $this->ifEquals( $value, 'style' ) || $this->ifEquals( $value, 'code' ) )
		{
			return true;
		}
		return false;
	}

	function forward( $string, $n )
	{
		if( is_null( $this->TAG_HELPER ) )
		{
			return true;
		}
		
		else
		{
			$count = strlen( $this->TAG_HELPER );
			$buffer = '';
			$i = 1;

			while( $i <= $count )
			{
				$buffer .= $string[ $n + $i ];
				$i++;
			}

			if( !empty( $buffer ) && $this->ifEquals( $buffer, $this->TAG_HELPER ) )
			{
				return true;
			}			
		}
		return false;
	}

	function searchWpadminbar()
	{
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( !empty( $tag['attributes'] ) )
			{
				foreach( $tag['attributes'] as $attribute )
				{
					if( $this->ifEquals( $attribute[0], 'id' ) )
					{
						if( !empty( $attribute[1] ) && $this->ifEquals( $attribute[1], 'wpadminbar' ) )
						{
							return $tag['path'].'.'.$i;
						}
					}
				}
			}
		}
		return '';
	}

	function getSegments()
	{
		$path = $this->searchWpadminbar();
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( !$this->ifSegment( $tag['name'] ) && !$this->ifEquals( $tag['name'], 'title' ) )
			{
				if( !empty( $tag['segments'] ) )
				{
					if( !empty( $path ) )
					{
						if( strncmp( $path, $tag['path'], strlen( $path ) ) !== 0 )
						{
							foreach( $tag['segments'] as $segment )
							{
								if( !empty( $segment ) )
								{
									$result[] = $segment;
								}
							}
						}
					}

					else
					{
						foreach( $tag['segments'] as $segment )
						{
							if( !empty( $segment ) )
							{
								$result[] = $segment;
							}
						}
					}
				}
			}
		}
		return $result;
	}

	function getImages()
	{
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( $this->ifEquals( $tag['name'], 'img' ) )
			{
				if( !empty( $tag['attributes'] ) )
				{
					foreach( $tag['attributes'] as $attribute )
					{
						if( $this->ifEquals( $attribute[0], 'src' ) )
						{
							if( !empty( $attribute[1] ) )
							{
								$result[] = $attribute[1];
							}
						}
					}
				}
			}
		}
		return $result;
	}

	function getLinks()
	{
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( $this->ifEquals( $tag['name'], 'a' ) )
			{
				if( !empty( $tag['attributes'] ) )
				{
					foreach( $tag['attributes'] as $attribute )
					{
						if( $this->ifEquals( $attribute[0], 'href' ) )
						{
							if( !empty( $attribute[1] ) )
							{
								$result[] = $attribute[1];
							}
						}
					}
				}
			}
		}
		return $result;
	}

	function getAttributes()
	{
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( !empty( $tag['attributes'] ) )
			{
				foreach( $tag['attributes'] as $attribute )
				{
					if( $this->ifEquals( $attribute[0], 'title' ) || $this->ifEquals( $attribute[0], 'alt' ) || $this->ifEquals( $attribute[0], 'placeholder' ) )
					{
						if( !empty( $attribute[1] ) )
						{
							$result[] = $attribute[1];
						}
					}
				}
			}
		}
		return $result;
	}

	function getMeta()
	{
		$result = array();

		$n = sizeof( $this->data );

		for($i = 0; $i < $n; $i++)
		{
			$tag = $this->data[$i];

			if( $this->ifEquals( $tag['name'], 'meta' ) )
			{
				if( !empty( $tag['attributes'] ) )
				{
					foreach( $tag['attributes'] as $attribute )
					{
						if( $this->ifEquals( $attribute[0], 'name' ) )
						{
							if( $this->ifEquals( $attribute[1], 'description' ) || $this->ifEquals( $attribute[1], 'keywords' ) )
							{
								foreach( $tag['attributes'] as $temp )
								{
									if( $this->ifEquals( $temp[0], 'content' ) )
									{
										if( !empty( $temp[1] ) )
										{
											$result[] = $temp[1];
											//$result[] = $temp[1];
										}
									}
								}
							}
						}
					}
				}
			}

			if( $this->ifEquals( $tag['name'], 'title' ) )
			{
				if( !empty( $tag['segments'] ) )
				{
					foreach( $tag['segments'] as $segment )
					{
						if( !empty( $segment ) )
						{
							$result[] = $segment;
						}
					}
				}
			}
		}
		return $result;
	}

	function read( $string )
	{
		$this->THIS_OPENING_TAG = false;
		$this->WAITING_TAG_NAME = false;
		$this->WAITING_ATTRIBUTE_NAME = false;
		$this->WAITING_QUOTE_HELPER = false;
		$this->WAITING_ATTRIBUTE_VALUE = false;
		$this->WAITING_SEGMENT = false;

		$n = strlen( $string );

		for ($i = 0; $i < $n; $i++)
		{
			$char = $string[$i];

			if( $this->WAITING_ATTRIBUTE_NAME )
			{
				if( $this->ifSpace( $char ) )
				{
					continue;
				}

				if( $char === $this->SYMBOL_RIGHT_QUOTE )
				{
					$this->addAttribute( $this->buffer );
					$this->bufferReset();

					$this->WAITING_ATTRIBUTE_NAME = false;
					$this->WAITING_SEGMENT = true;

					continue;
				}

				if( $char === $this->SYMBOL_EQUALS )
				{
					$this->addAttribute( $this->buffer );
					$this->bufferReset();

					$this->WAITING_QUOTE_HELPER = true;
					$this->WAITING_ATTRIBUTE_NAME = false;

					continue;					
				}
			}

			if( $this->WAITING_QUOTE_HELPER )
			{
				if( $char === $this->SYMBOL_DOUBLE_QUOTES )
				{
					$this->QUOTE_HELPER = $this->SYMBOL_DOUBLE_QUOTES;
					$this->WAITING_QUOTE_HELPER = false;
					$this->WAITING_ATTRIBUTE_VALUE = true;

					continue;
				}

				if( $char === $this->SYMBOL_SINGLE_QUOTE )
				{
					$this->QUOTE_HELPER = $this->SYMBOL_SINGLE_QUOTE;
					$this->WAITING_QUOTE_HELPER = false;
					$this->WAITING_ATTRIBUTE_VALUE = true;

					continue;
				}
			}
			
			if( $this->WAITING_ATTRIBUTE_VALUE )
			{
				if( $char === $this->QUOTE_HELPER )
				{
					$this->addAttrValue( $this->buffer );
					$this->bufferReset();

					$this->WAITING_ATTRIBUTE_VALUE = false;
					$this->WAITING_ATTRIBUTE_NAME = true;

					continue;					
				}
			}

			if( $this->WAITING_TAG_NAME )
			{
				if( $this->ifSpace( $char ) )
				{
					if( empty( $this->buffer ) )
					{
						//die( 'ERROR TAG NAME '. $i );
					}
					$this->addTag( $this->buffer );
					$this->bufferReset();

					$this->WAITING_TAG_NAME = false;
					$this->THIS_OPENING_TAG = false;

					$this->WAITING_ATTRIBUTE_NAME = true;

					continue;
				}

				if( $char === $this->SYMBOL_SLASH )
				{
					$this->THIS_OPENING_TAG = false;

					continue;
				}

				if( $char === $this->SYMBOL_RIGHT_QUOTE )
				{
					$this->addTag( $this->buffer );
					$this->bufferReset();

					$this->WAITING_TAG_NAME = false;
					$this->THIS_OPENING_TAG = false;					
					$this->WAITING_SEGMENT = true;

					continue;
				}
			}

			if( $char === $this->SYMBOL_LEFT_QUOTE )
			{
				if( $this->forward( $string, $i ) )
				{
					$this->WAITING_TAG_NAME = true;
					$this->THIS_OPENING_TAG = true;

					if( $this->WAITING_SEGMENT )
					{
						$this->prepareSegment( $this->buffer );
						$this->bufferReset();

						$this->WAITING_SEGMENT = false;
					}

					continue;
				}
			}

			$this->buffer .= $char;
		}
	}
}
